﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.SignalR;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;

namespace Tsi1.Api.SignalR
{
    [Authorize]
    public class VideoHub : Hub
    {
        private readonly IActivityService _activityService;

        public VideoHub(IActivityService activityService)
        {
            _activityService = activityService;
        }

        public override async Task OnConnectedAsync()
        {
            await Groups.AddToGroupAsync(Context.ConnectionId, Context.ConnectionId);

            await Clients.Caller.SendAsync(Context.ConnectionId);
        }

        [Authorize(Roles = UserTypes.Professor)]
        public async Task CreateRoom(int activityId)
        {
            var result = await _activityService.ActivityValidation(activityId);

            if (result.HasError)
            {
                throw new HubException(result.Message);
            }

            await Groups.AddToGroupAsync(Context.ConnectionId, activityId.ToString());

            await Clients.Caller.SendAsync("CreateRoom", activityId);
        }

        public async Task Join(int activityId, string offer)
        {
            var userId = int.Parse(Context.User.Claims.FirstOrDefault(x => x.Type == "Id").Value);

            var result = await _activityService.AttendVirtualClass(activityId, userId);

            if (result.HasError)
            {
                throw new HubException(result.Message);
            }

            await Clients.Group(activityId.ToString()).SendAsync("Join", Context.ConnectionId, offer);

            await Groups.AddToGroupAsync(Context.ConnectionId, activityId.ToString());
        }

        public async Task Leave(string roomName)
        {
            await Groups.RemoveFromGroupAsync(Context.ConnectionId, roomName);

            await Clients.Group(roomName).SendAsync("Leave", Context.ConnectionId);
        }

        public async Task NotifyUser(string connectionId, string answer)
        {
            await Clients.Group(connectionId).SendAsync("NotifyUser", Context.ConnectionId, answer);
        }

        public async Task ICECandidate(string connectionId, string candidate)
        {
            await Clients.Group(connectionId).SendAsync("ICECandidate", Context.ConnectionId, candidate);
        }
    }
}
